//----------------------------------------------------
//  AD7992.C
//
// 08.06.08.2005
// Robert Tyrakowski, DK7NT
//----------------------------------------------------

#include <type.h>
#include <AD7992.h>
#include <lpc214x.h>

#include <psys.h>

extern u_int hardware_status;

void AD7992_error(void);
u_int Init_AD7992_Config_X(void);
u_int Read_AD7992_X(u_int bytecount);
u_int AD7992_Result_X(void);
u_int Select_ConversationRegister(void);

u_char AD7992_Buf[3];

//--------------------------------------------------------
//           AD7992 - Read Bytwe
//
//   returns: value
//   (no error return)
//--------------------------------------------------------

u_int AD7992_Read_Byte(void)
{
u_int value,i;
   i = 8;
   value = 0;
   SDAADC_High;
   SDAADC_High;
   SDAADC_DIR_IN;
	do {
     	SCLADC_High;
     	SCLADC_High;
      value <<= 1;
      value |= SDAADC_IN;
 	   SCLADC_Low;
 	   SCLADC_Low;
   }
   while( --i );
   SDAADC_DIR_OUT;
   return ( value );
}

//--------------------------------------------------------
//           AD7992 - Write Byte
//
//	  datum = value to write
//   returns:  0 ok, 1 not ok
//
//--------------------------------------------------------

u_int AD7992_Write_Byte(u_char datum)
{
u_int i = 8;
   SDAADC_DIR_OUT;
   do {
	   if ( datum & 0x80 ) {
         SDAADC_High;
         SDAADC_High;
      }
      else  {
         SDAADC_Low;
         SDAADC_Low;
      }
      datum<<=1;
      SCLADC_High;
	   SCLADC_High;
      SCLADC_Low;
      SCLADC_Low;
	}
	while( --i );
   SDAADC_DIR_IN;
	SDAADC_Low;      // read ACK
	SDAADC_Low;      // read ACK
   SCLADC_High;
   SCLADC_High;
	i = SDAADC_IN;
   SCLADC_Low;
   SCLADC_Low;
   SDAADC_DIR_OUT;
   return(i);
}
//--------------------------------------------------------
//                 I2C  - Write Acknowledge
//--------------------------------------------------------

void AD7992_Write_Ack(void)
{
   SDAADC_DIR_OUT;
	SDAADC_Low;
	SDAADC_Low;
   SCLADC_High;
   SCLADC_High;
   SCLADC_Low;
   SCLADC_Low;
   SDAADC_DIR_IN;
	SDAADC_High;
	SDAADC_High;
}

//----------------------------------------------------------------------------
//                 I2C - Write Slave Address
//   rw : 1 = read , 0 write
//   returns:  0 ok, 1 not ok
//----------------------------------------------------------------------------
u_int AD7992_Write_SlaveAddress(u_int rw)
{
   AD7992_Start();
  	return (AD7992_Write_Byte(AD7992_addr << 1 | rw));
}

//----------------------------------------------------------------------------
//        Stop AD7992
//----------------------------------------------------------------------------

void AD7992_Stop(void)
{
   SDAADC_Low;			// stop
   SDAADC_Low;			// stop
 	SCLADC_High;
 	SCLADC_High;
   SDAADC_High;
   SDAADC_High;
   SDAADC_High;
}

//----------------------------------------------------------------------------
//        Start AD7992
//----------------------------------------------------------------------------

void AD7992_Start(void)
{
   SDAADC_DIR_OUT;
 	SCLADC_High;
 	SCLADC_High;
   SDAADC_High;
   SDAADC_High;

   SDAADC_Low;
   SDAADC_Low;
 	SCLADC_Low;
 	SCLADC_Low;
}

/*
-----------------------------------------------------------------------------
   2 channel ADC  AD7992
	bytecount is number of byte to read : 1 or 2
	Read result
   returns:  0 ok, 1 not ok
-----------------------------------------------------------------------------
*/
u_int Read_AD7992(u_int bytecount)
{
u_int notok = 1;
u_int try_again = try_again_set;
   notok = Read_AD7992_X(bytecount);
	if (notok) {
	   while (--try_again && notok) {
		   notok = Read_AD7992_X(bytecount);
		}
      if (!try_again) {
	      AD7992_error();
		}
   }
   return (notok);
}

u_int Read_AD7992_X(u_int bytecount)
{
u_int notok = 1;
u_int i = 0;
   if (!(AD7992_Write_SlaveAddress(rd))) {              // for read
      AD7992_Buf[i++]=AD7992_Read_Byte() ;
      if (bytecount == 2) {
		   AD7992_Write_Ack();
		   AD7992_Buf[i]=AD7992_Read_Byte() ;
	 	}
		notok = 0;
   }
   AD7992_Stop();                                      //i2c stop
   return(notok);
}
//-----------------------------------------------------------------------------
// AD7992 configuration
//
// returns:  0 ok, 1 not ok
//-----------------------------------------------------------------------------
u_int Init_AD7992_Config(void)
{
u_int notok = 1;
u_int try_again = try_again_set;
   notok = Init_AD7992_Config_X();
	if (notok) {
	   while (--try_again && notok) {
		   notok = Init_AD7992_Config_X();
		}
      if (!try_again) {
	      AD7992_error();
		}
   }
	return (notok);
}

u_int Init_AD7992_Config_X(void)
{
u_int notok = 1;
   if (!(AD7992_Write_SlaveAddress(wr))) {              // for write
	   if (!(AD7992_Write_Byte(AD7992_convreg))) {
	      if (!(AD7992_Write_Byte(AD7992_config))) {
            notok = 0;
		   }
		}
	}
   AD7992_Stop();              							     //i2c stop
   return(notok);
}

//-----------------------------------------------------------------------------
// AD7992 read result
//
// returns:  0 ok, 1 not ok
//-----------------------------------------------------------------------------
u_int AD7992_Result(void)
{
u_int notok = 1;
u_int try_again = try_again_set;
   notok = AD7992_Result_X();
	if (notok) {
	   while (--try_again && notok) {
		   notok = AD7992_Result_X();
		}
      if (!try_again) {
	      AD7992_error();
		}
   }
   return (notok);
}

u_int AD7992_Result_X(void)
{
u_int notok = 1;
   if (!(AD7992_Write_SlaveAddress(wr))) {              // for write
	   if (!(AD7992_Write_Byte(AD7992_convres))) {
         AD7992_Stop();
	      if (!(Read_AD7992(2))) {
            notok = 0;
			}
		}
	}
   AD7992_Stop();              							     //i2c stop
   return(notok);
}

//-------------------------------------------------------------------------
void AD7992_error(void)
{
	hardware_status &= ~ad7992_ok;
}

//-------------------------------------------------------------------------
// start conversion and read result
// result is in AD7992_Buf[]
// returns 0 for ok, 1 for not ok
//-------------------------------------------------------------------------
u_int AD7992_Convert(void)
{
u_int x;
   AD7992_Buf[0]= 0;
   AD7992_Buf[1]= 0;
   AD7992_Buf[2]= 0;
   AD7992_CONVST_high;
   AD7992_CONVST_high;
//   AD7992_CONVST_high;
   AD7992_CONVST_low;
   AD7992_CONVST_low;
//   AD7992_CONVST_low;
   x = AD7992_Result();
   return x;
}

