/*
------------------------------
  AD8556.C

  20.05.2006
  Robert Tyrakowski, DK7NT

------------------------------

0 : ___|-------------|______________________________|-
        50 ns ..10 us  Time between pulses : >10 us

1 : ___|-------------------------------------|____________
        > 50 us

AD8556 Record
-------------
 bit
00--11   12-Bit Start of Packet 1000 0000 0001
12--13   Function               xx
14--15   Parameter              xx
16--17   Dummy                  10
18--25   Value                  xxxx xxxx
26--37   12-Bit Stop of Packet  0111 1111 1110

Multiplier = first stage * second stage
*/

#include <lpc214x.h>
#include <type.h>
#include <ad8556.h>
//#include <pyrano.h>

extern void wait(u_long time);
extern u_int time_over;


float ad8556_gain;
float ad8556_gain_real;
u_int gain1;
u_int gain2;
u_int ad8556_offset;
long ad8556_offset_temp;

const float first_stage[] = {
   4.000,4.015,4.030,4.045,4.060,4.075,4.090,4.105,   //   0..  7
	4.120,4.135,4.151,4.166,4.182,4.197,4.213,4.228,   //   8.. 15
	4.244,4.260,4.276,4.291,4.307,4.323,4.339,4.355,   //  16.. 23
	4.372,4.388,4.404,4.420,4.437,4.453,4.470,4.486,   //  24.. 31
	4.503,4.520,4.536,4.553,4.570,4.587,4.604,4.621,   //  32.. 39
	4.638,4.655,4.673,4.690,4.707,4.725,4.742,4.760,   //  40.. 47
	4.778,4.795,4.813,4.831,4.849,4.867,4.885,4.903,   //  48.. 55
   4.921,4.939,4.958,4.976,4.995,5.013,5.032,5.050,   //  56.. 63
	5.069,5.088,5.107,5.126,5.145,5.164,5.183,5.202,   //  64.. 71
	5.221,5.241,5.260,5.280,5.299,5.319,5.339,5.358,   //  72.. 79
	5.378,5.398,5.418,5.438,5.458,5.479,5.499,5.519,   //  80.. 87
	5.540,5.560,5.581,5.602,5.622,5.643,5.664,5.685,   //  88.. 95
	5.706,5.727,5.749,5.770,5.791,5.813,5.834,5.856,   //  96..103
	5.878,5.900,5.921,5.943,5.965,5.988,6.010,6.032,   // 104..111
   6.054,6.077,6.099,6.122,6.145,6.167,6.190,6.213,   // 112..119
   6.236,6.259,6.283,6.306,6.329,5.353,5.376,6.400    // 120..127
	};

const float second_stage[] = {
   17.5,25.0,35.0,50.0,70.0,100.0,140.0,200.0         //  0.. 7
	};

// Set  bit to 0 in AD8556
void AD8556_0(void)
{
   ad8556_high ;     //
   ad8556_high ;     //
   ad8556_low ;
   wait(11);  	 		// wait 16 us
	while (!time_over);
}
// Set bit to 1 in AD8556
void AD8556_1(void)
{
   ad8556_high ;
   wait(60);  	 		// wait 60 us
	while (!time_over);
   ad8556_low ;
   wait(11);  	 		// wait 16 us
	while (!time_over);
}

// Transfer 38 bit to the AD8556- MSB first
void AD8556_38(u_int function, u_int parameter, u_int value)
{
u_int i,a;
   a = ad8556_start;	  		  //Start
   for (i=0;i<12;i++)
	{
	   if (a & 0x0800)
		   AD8556_1();
		else
		   AD8556_0();
		a<<=1;
	}
	a = function;
   for (i=0;i<2;i++)         // Function
	{
	   if (a & 0x0002)
		   AD8556_1();
		else
		   AD8556_0();
		a<<=1;
	}
   a = parameter;
	for (i=0;i<2;i++)			  // Parameter
	{
	   if (a & 0x0002)
		   AD8556_1();
		else
		   AD8556_0();
		a<<=1;
	}
	a = 0x2;
   for (i=0;i<2;i++)			  // Dummy
	{
	   if (a & 0x0002)
		   AD8556_1();
		else
		   AD8556_0();
		a<<=1;
	}
	a = value;
   for (i=0;i<8;i++)			  // Value
	{
	   if (a & 0x0080)
		   AD8556_1();
		else
		   AD8556_0();
		a<<=1;
	}
   a = ad8556_stop;
   for (i=0;i<12;i++)        // Stop
	{
	   if (a & 0x0800)
		   AD8556_1();
		else
		   AD8556_0();
		a<<=1;
	}
}

u_int Calculate_Gain(void)
{
float x,y,y1,y2,v2;
u_int i;

   gain2 = 0;  // 17.5
	gain1 = 0;  // 4
   if (ad8556_gain < 70) return FALSE;
	if (ad8556_gain > 1280) return FALSE;
	if (ad8556_gain < 100) {
	   ;                                // 70 .. 99
	}
	else {
	   if (ad8556_gain < 140) {
		   gain2 = 1; 		               // 100 .. 139
		}
		else {
 	      if (ad8556_gain < 200) {
		      gain2 = 2; 		            // 140 .. 199
		   }
		   else {
 	         if (ad8556_gain < 280) {
		         gain2 = 3;              // 200 .. 279
		      }
		      else {
 	            if (ad8556_gain < 400) {
		            gain2 = 4; 			   // 280 .. 399
		         }
		         else {
 	               if (ad8556_gain < 560) {
		               gain2 = 5; 		   // 400 .. 559
		            }
		            else {
 	                  if (ad8556_gain < 800) {
		                  gain2 = 6; 		// 560 .. 799
		               }
							else {
		                  gain2 = 7; 		// 800 .. 1280
							}
                  }
					}
				}
			}
		}
	}
	v2 = second_stage[gain2];
	x = ad8556_gain/v2;
	y1 = first_stage[0];
	for (i=1;i<128;i++) {
	   y2 = first_stage[i];
		if (x < y2) break;
		y1 = y2;
	}
	i--;
	y = x-y1;
	if ((y2 - x) > y) i--;
	gain1 = i;
	ad8556_gain_real = v2 * first_stage[i];
	return TRUE;
}

// setup AD8556 gain and Offset
void AD8556_Setup(void)
{
	AD8556_38(ad8556_func_simulate, ad8556_set_first_stage, gain1);
	AD8556_38(ad8556_func_simulate, ad8556_set_second_stage, gain2);
	AD8556_38(ad8556_func_simulate, ad8556_set_offset, ad8556_offset);
}

// Init AD8556 - Set default value
u_int AD8556_Init(void)
{
	if (!Calculate_Gain()) return FALSE ;
   AD8556_Setup();
	return TRUE;
}
